//============================================================================
//  aPCI-8806 for PCI Device Driver Diagnosis program
//  apci8806_diag.c
//  Author  Shoichiro Ito(ADTEK SYSTEM SCIENCE Co.,Ltd.)
//  Date    2001/03/30
//  Rev.    1.0.0
//  Note
//============================================================================

#include <stdio.h>	// used printf,perror...
#include <stdlib.h>	// used malloc,free
#include <unistd.h>	// used read
#include <sys/ioctl.h>
#include <term.h>
#include <string.h>	// used strerror
#include <errno.h>	// used errno

#include <math.h>	// used sin

#include "apci8806ctl.h"


// input of command from user
static char line[256];


//--- Ǵؿ ----------------------------------------------

static void diag01_create(void);
static void diag02_close(void);
static void diag03_set_da_data(void);
static void diag04_get_exin(void);

static void diag05_test_smp(void);
static void diag06_test_exin(void);

int make_square(unsigned short* pdata);
int make_triangle(unsigned short* pdata);
int make_sine(unsigned short* pdata);

static int    enter_log_slot(void);
static struct termio init_termio(void);
static void   term_termio(struct termio tty);


//--- ؿơ֥ ------------------------------------------

static void (*pdiag_func[])(void) =
{
	diag01_create,
	diag02_close,
	diag03_set_da_data,
	diag04_get_exin,
	diag05_test_smp,
	diag06_test_exin
};


int main(int argc, char *argv[])
{
	int cmd, i;

	printf("aPCI-8806 diagnostic utility.\n");

	do
	{
		printf("\n");
		printf("aPCI-8806 Test main menu\n");
		printf("------------------------\n");
		printf(" Function test          \n");	// ǽƥ
		printf("------------------------\n");
		printf(" 1. apci8806_create(int log_slot)                           \n");
		printf(" 2. apci8806_close(int log_slot)                            \n");
		printf(" 3. apci8806_set_da_data(int log_slot, unsigned short ch, unsigned short data)\n");
		printf(" 4. apci8806_get_exin(int log_slot, unsigned char* pexin)   \n");
		printf("------------------------\n");
		printf(" Application test       \n");	// ѥƥ
		printf("------------------------\n");
		printf(" 5. D/A Sampling                                           \n");
		printf(" 6. Outside input                                          \n");
		printf("------------------------\n");
		printf("99. Exit (All devices are closed)\n");
		printf("Enter option: ");

		cmd = 0;
		fgets(line, sizeof(line), stdin);
		sscanf(line, "%d", &cmd);

		if (cmd > 0 && cmd <= sizeof(pdiag_func) / sizeof(void*))
			pdiag_func[cmd - 1]();

		if(cmd != 99)
		{
			printf("--- Please enter key push! ---");
			fgets(line, sizeof(line), stdin);
		}
	}
	while (cmd != 99);

	// ƤΥǥХ򥯥
	for (i = 0; i < APCI8806_MAX_DEVICES; i++)
		apci8806_close(i);

	return 0;
}

/*--------------------------------------------------------------------------*/
/*  ǥХΥץ							    */
/*--------------------------------------------------------------------------*/
static void diag01_create(void)
{
	int log_slot;

	printf("--- apci8806_create(int log_slot)\n");

	// Get LogSlot
	printf("Enter logslot(0-15 or -1): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &log_slot);

	// Call Control Function
	log_slot = apci8806_create(log_slot);

	// result
	if (log_slot < 0)
		printf("Rslt: %d\n", log_slot);
	else
		printf("log_slot: %d\n", log_slot);
}

/*--------------------------------------------------------------------------*/
/*  ǥХΥ							    */
/*--------------------------------------------------------------------------*/
static void diag02_close(void)
{
	int log_slot;
	int rslt;

	printf("--- apci8806_close(int log_slot)\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Call Control Function
	rslt = apci8806_close(log_slot);

	printf("Rslt: %d\n", rslt);
}

/*--------------------------------------------------------------------------*/
/*  D/A ǡ							    */
/*--------------------------------------------------------------------------*/
static void diag03_set_da_data(void)
{
	int log_slot;
	int rslt, ent;
	unsigned short ch, data;

	printf("--- apci8806_set_da_data(int log_slot, unsigned short ch, unsigned short data)\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Get Channel
	printf("Enter Channel(0-3): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &ent);
	ch = (unsigned short)ent;

	// Get Data
	printf("Enter Data(hex): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%x", &ent);
	data = (unsigned short)ent;

	// Call Control Function
	rslt = apci8806_set_da_data(log_slot, ch, data);

	printf("Rslt: %d\n", rslt);
}

/*--------------------------------------------------------------------------*/
/*  ϥݡȤǡ					    */
/*--------------------------------------------------------------------------*/
static void diag04_get_exin(void)
{
	int log_slot;
	int rslt;
	unsigned char exin;

	printf("--- apci8806_get_exin(int log_slot, unsigned char* pexin)\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	rslt = apci8806_get_exin(log_slot, &exin);

	printf("Rslt: %d\n", rslt);
	printf("exin(hex): %x\n", exin);
}

/*--------------------------------------------------------------------------*/
/*  D/A ǡϢ³						    */
/*--------------------------------------------------------------------------*/
static void diag05_test_smp(void)
{
	int log_slot;
	int mode;
	int rslt, ent;
	char   c, n;
	struct termio tty_save;
	unsigned short ch;
	unsigned short* pdata;
	int i, count = 0;

	printf("--- The continuous test of D/A Sampling\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Get Channel
	printf("Enter Channel(0-3): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &ent);
	ch = (unsigned short)ent;

	// Get Mode
	do
	{
		ent = 0;
		printf("Enter Mode(1:Square 2:Triangle 3:Sine): ");
		fgets(line, sizeof(line), stdin);
		sscanf(line, "%d", &ent);
		mode = ent;
	}
	while (!(mode == 1 || mode == 2 || mode == 3));

	pdata = malloc(256 * sizeof(unsigned short));
	if (pdata == NULL)
	{
		printf("Memory allocation error!\n");
		return;
	}

	switch (mode)
	{
		case 1:
			count = make_square(pdata);
			break;
		case 2:
			count = make_triangle(pdata);
			break;
		case 3:
			count = make_sine(pdata);
			break;
	}

	printf("\nA completion key is \"ESC\".\n\n");

	tty_save = init_termio();

	for (i = 0;; i++)
	{
		if (i >= count)
			i = 0;

		c = '\0';		// ʤ硢ƤѤǤޤ
		n = read(0, &c, 1);
		if (n == -1)
		{
			perror("read");
			break;
		}
		else if (c == 0x1B)	// ESC key
			break;

		// Call Control Function
		rslt = apci8806_set_da_data(log_slot, ch, pdata[i]);
		if (rslt < 0)
		{
			printf(" Rslt: %d ", rslt);
			printf("\r");
			fflush(stdout);
			continue;
		}
	}

	// ᤹
	term_termio(tty_save);

	printf("\n");

	free(pdata);
}

/*--------------------------------------------------------------------------*/
/*  ϥݡȤǡϢ³					    */
/*--------------------------------------------------------------------------*/
static void diag06_test_exin(void)
{
	int log_slot;
	int rslt;
	char   c, n;
	struct termio tty_save;
	unsigned char exin;

	printf("--- The continuous test of Outside-input\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	printf("\nA completion key is \"ESC\".\n\n");

	printf(" EXIN1 EXIN2\n");
	printf("-------------\n");

	tty_save = init_termio();

	for (;;)
	{
		c = '\0';		// ʤ硢ƤѤǤޤ
		n = read(0, &c, 1);
		if (n == -1)
		{
			perror("read");
			break;
		}
		else if (c == 0x1B)	// ESC key
			break;

		// Call Control Function
		rslt = apci8806_get_exin(log_slot, &exin);
		if (rslt < 0)
		{
			printf(" Rslt: %d ", rslt);
			printf("\r");
			fflush(stdout);
			continue;
		}

		// ϥݡ CH1 ξ
		printf("   %d  ", exin & APCI8806_EXIN1 ? 1 : 0);

		// ϥݡ CH2 ξ
		printf("   %d  ", exin & APCI8806_EXIN2 ? 1 : 0);

		printf("\r");
		fflush(stdout);
	}

	// ᤹
	term_termio(tty_save);

	printf("\n");
}

/*--------------------------------------------------------------------------*/
/*  ȥǡκ							    */
/*--------------------------------------------------------------------------*/
int make_square(unsigned short* pdata)
{
	int i;

	for (i = 0; i < 64; i++)
		pdata[i] = 0x000;

	for (; i < 128; i++)
		pdata[i] = 0xFFF;

	return i;
}

/*--------------------------------------------------------------------------*/
/*  ȥǡκ							    */
/*--------------------------------------------------------------------------*/
int make_triangle(unsigned short* pdata)
{
	int i;
	unsigned short data;

	for (i = 0, data = 0; i < 64; i++, data += 64)
		pdata[i] = data;

	for (; i < 128; i++, data -= 64)
	{
		// Υǡݡ 0xFFF ۤƤΤǡ
		// Ƥޤ
		if (data > 0xFFF)
			pdata[i] = 0xFFF;
		else
			pdata[i] = data;
	}

	return i;
}

/*--------------------------------------------------------------------------*/
/*  ȥǡκ							    */
/*--------------------------------------------------------------------------*/
int make_sine(unsigned short* pdata)
{
	int i, data;

	for (i = 0, data = 0; i < 120; i++, data += 3)
		pdata[i] = (unsigned short)(((double)0x7ff * sin((double)data / 180.0 * 3.14159)) + 0x800);

	return i;
}

/*--------------------------------------------------------------------------*/
/*  åȤ							    */
/*--------------------------------------------------------------------------*/
static int enter_log_slot(void)
{
	int log_slot;

	printf("Enter log_slot: ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &log_slot);

	return log_slot;
}

/*--------------------------------------------------------------------------*/
/*  ꥢ륿७						    */
/*--------------------------------------------------------------------------*/
static struct termio init_termio(void)
{
	struct termio tty, tty_save;

	// ߤ¸
	ioctl(0, TCGETA, &tty);

	tty_save = tty;

	// ü
	tty.c_lflag &= ~(ECHO | ICANON);
	tty.c_cc[VMIN] = 0;
	tty.c_cc[VTIME] = 2;
	ioctl(0, TCSETAF, &tty);

	return tty_save;
}

/*--------------------------------------------------------------------------*/
/*  ꥢ륿७ϲ						    */
/*--------------------------------------------------------------------------*/
static void term_termio(struct termio tty)
{
	// ᤹
	ioctl (0, TCSETAF, &tty);
}
